@@ -45,7 +45,13 @@
  * Acrobat 4 is now more than 20 years old, lets just drop support for it. The
  * PDF specification never had this limit, just Adobe's software.
  */
-/* #define MAX_USER_COORD 16300 */
+/* 31/05/2021 However, it transpires that the PDF/A-1 specification adopted the
+ * implementation limits of Acrobat 5 as part of the specification. So we need
+ * to handle that, but we don't need to limit ourselves to the more restrictive
+ * Acrobat 4 limit of 16384 ( MAX_USER_COORD 16300), we can use 32767. THis is only
+ * used when creating PDF/A-1 files.
+ */
+#define MAX_USER_COORD 32700
 
 /* ---------------- Statically allocated sizes ---------------- */
 /* These should really be dynamic.... */
@@ -946,6 +952,7 @@ struct gx_device_pdf_s {
     gs_char OCR_char_code;          /* Passes the current character code from text processing to the image processing code when rendering glyph bitmaps for OCR */
     gs_glyph OCR_glyph;             /* Passes the current glyph code from text processing to the image processing code when rendering glyph bitmaps for OCR */
     ocr_glyph_t *ocr_glyphs;        /* Records bitmaps and other data from text processing when doing OCR */
+    gs_gstate **initial_pattern_states;
 };
 
 #define is_in_page(pdev)\
@@ -1019,7 +1026,7 @@ dev_proc_fill_rectangle_hl_color(gdev_pdf_fill_rectangle_hl_color);
     /* In gdevpdfv.c */
 dev_proc_include_color_space(gdev_pdf_include_color_space);
     /* In gdevpdft.c */
-dev_proc_create_compositor(gdev_pdf_create_compositor);
+dev_proc_composite(gdev_pdf_composite);
 dev_proc_begin_transparency_group(gdev_pdf_begin_transparency_group);
 dev_proc_end_transparency_group(gdev_pdf_end_transparency_group);
 dev_proc_begin_transparency_mask(gdev_pdf_begin_transparency_mask);
